#include "OTAProxy.h"
#include "NFC.h"
#include <PubSubClient.h>
#include <ArduinoLog.h>
#include "helpers.h"

OTAProxy::OTAProxy(PubSubClient *mqttClient, NFC *nfc, int id)
{
    this->mqtt = mqttClient;
    this->nfc = nfc;
    this->id = id;
}

bool OTAProxy::hasActiveOTA()
{
    return activeOTA;
}

void OTAProxy::startOTA()
{
    if (!(nfc->hasCardSelected()))
    {
        return;
    }
    activeOTA = true;

    char topic[] = "fabreader/00000/startOTA";
    sprintf(topic, "fabreader/%05d/startOTA", id);

    String uid = nfc->getUID();
    mqtt->publish(topic, uid.c_str(), uid.length());

    Log.info(F("Start OTA"));
}

void OTAProxy::continueOTA(char *topic, byte *payload, unsigned int length)
{
    // Create topic strings with sufficient buffer sizes
    char topic_requestOTA[] = "fabreader/00000/requestOTA";
    sprintf(topic_requestOTA, "fabreader/%05d/requestOTA", id);

    char topic_responseOTA[] = "fabreader/00000/responseOTA";
    sprintf(topic_responseOTA, "fabreader/%05d/responseOTA", id);

    char topic_stopOTA[] = "fabreader/00000/stopOTA";
    sprintf(topic_stopOTA, "fabreader/%05d/stopOTA", id);

    char topic_restartOTA[] = "fabreader/00000/restartOTA";
    sprintf(topic_restartOTA, "fabreader/%05d/restartOTA", id);

    if (!strcmp(topic, topic_requestOTA))
    {
        Log.trace("Request OTA");
        byte response[APDU_BUFFER_SIZE] = {0};
        byte response_len;

        uint8_t status;

        Log.trace("Run Transceive");

        Log.trace("Payload length: %d", length);
        Log.trace("Payload: %X", payload);

        status = nfc->Transceive(payload, length, response, &response_len);
        Serial.printf("PICC_Tranceive: 0x%02x\n", status);

        if (status != 0x00)
        {
            cancelOTA();
            return;
        }

        Log.trace("Response length: %d", response_len);
        Log.trace("Response: %X", response);

        printbytes(response, response_len);

        bool mqttResult = mqtt->publish(topic_responseOTA, response, response_len);

        if (!mqttResult)
        {
            Log.error("MQTT publish failed");
            cancelOTA();
            return;
        }

        Log.trace("Response OTA");
    }
    else if (!strcmp(topic, topic_stopOTA))
    {
        Log.trace("Stop OTA");
        nfc->disconnectCard();
        activeOTA = false;
    }
    // else if(!strcmp(topic, topic_restartOTA))
    // {
    //     Log.trace("Restart OTA");
    //     while(!(nfc->deselectCard()));
    //     // if(nfc->hasNewCard())
    //     // {
    //     //     startOTA();
    //     // }
    // }
}

void OTAProxy::cancelOTA()
{
    char topic_cancelOTA[] = "fabreader/00000/cancelOTA";
    sprintf(topic_cancelOTA, "fabreader/%05d/cancelOTA", id);

    String uid = nfc->getUID();
    mqtt->publish(topic_cancelOTA, uid.c_str(), uid.length());

    while (!(nfc->disconnectCard()))
    {
        yield();
    }
    activeOTA = false;

    Log.info("Cancel OTA");
}